@@ -11,166 +11,80 @@
 # available and licensed separately.  Please consult www.biosql.org
 """Implementations of Biopython-like Seq objects on top of BioSQL.
 
-This allows retrival of items stored in a BioSQL database using
+This allows retrieval of items stored in a BioSQL database using
 a biopython-like SeqRecord and Seq interface.
 
 Note: Currently we do not support recording per-letter-annotations
 (like quality scores) in BioSQL.
 """
 
-from Bio.Seq import Seq, UnknownSeq
+from Bio.Seq import Seq, SequenceDataAbstractBaseClass
 from Bio.SeqRecord import SeqRecord, _RestrictedDict
 from Bio import SeqFeature
 
 
-class DBSeq(Seq):
-    """BioSQL equivalent of the Biopython Seq object."""
+class _BioSQLSequenceData(SequenceDataAbstractBaseClass):
+    """Retrieves sequence data from a BioSQL database (PRIVATE)."""
 
-    def __init__(self, primary_id, adaptor, alphabet=None, start=0, length=0):
-        """Create a new DBSeq object referring to a BioSQL entry.
+    __slots__ = ("primary_id", "adaptor", "_length", "start")
 
-        You wouldn't normally create a DBSeq object yourself, this is done
-        for you when retrieving a DBSeqRecord object from the database.
+    def __init__(self, primary_id, adaptor, start=0, length=0):
+        """Create a new _BioSQLSequenceData object referring to a BioSQL entry.
+
+        You wouldn't normally create a _BioSQLSequenceData object yourself,
+        this is done for you when retrieving a DBSeqRecord object from the
+        database, which creates a Seq object using a _BioSQLSequenceData
+        instance as the data provider.
         """
-        if alphabet is not None:
-            raise ValueError("The alphabet argument is no longer supported")
         self.primary_id = primary_id
         self.adaptor = adaptor
         self._length = length
         self.start = start
+        super().__init__()
 
     def __len__(self):
         """Return the length of the sequence."""
         return self._length
 
-    def __getitem__(self, index):  # Seq API requirement
-        """Return a subsequence or single letter."""
-        if isinstance(index, int):
-            # Return a single letter as a string
-            i = index
+    def __getitem__(self, key):
+        """Return a subsequence as a bytes or a _BioSQLSequenceData object."""
+        if isinstance(key, slice):
+            start, end, step = key.indices(self._length)
+            size = len(range(start, end, step))
+            if size == 0:
+                return b""
+        else:
+            # Return a single letter as an integer (consistent with bytes)
+            i = key
             if i < 0:
-                if -i > self._length:
-                    raise IndexError(i)
-                i = i + self._length
+                i += self._length
+                if i < 0:
+                    raise IndexError(key)
             elif i >= self._length:
-                raise IndexError(i)
-            return self.adaptor.get_subseq_as_string(
+                raise IndexError(key)
+            c = self.adaptor.get_subseq_as_string(
                 self.primary_id, self.start + i, self.start + i + 1
             )
-        if not isinstance(index, slice):
-            raise TypeError("Unexpected index type")
+            return ord(c)
 
-        # Return the (sub)sequence as another DBSeq or Seq object
-        # (see the Seq obect's __getitem__ method)
-        if index.start is None:
-            i = 0
+        if step == 1:
+            if start == 0 and size == self._length:
+                # Return the full sequence as bytes
+                sequence = self.adaptor.get_subseq_as_string(
+                    self.primary_id, self.start, self.start + self._length
+                )
+                return sequence.encode("ASCII")
+            else:
+                # Return a _BioSQLSequenceData with the start and end adjusted
+                return _BioSQLSequenceData(
+                    self.primary_id, self.adaptor, self.start + start, size
+                )
         else:
-            i = index.start
-        if i < 0:
-            # Map to equavilent positive index
-            if -i > self._length:
-                raise IndexError(i)
-            i = i + self._length
-        elif i >= self._length:
-            # Trivial case, should return empty string!
-            i = self._length
-
-        if index.stop is None:
-            j = self._length
-        else:
-            j = index.stop
-        if j < 0:
-            # Map to equavilent positive index
-            if -j > self._length:
-                raise IndexError(j)
-            j = j + self._length
-        elif j >= self._length:
-            j = self._length
-
-        if i >= j:
-            # Trivial case, empty string.
-            return Seq("")
-        elif index.step is None or index.step == 1:
-            # Easy case - can return a DBSeq with the start and end adjusted
-            return self.__class__(
-                self.primary_id, self.adaptor, None, self.start + i, j - i
-            )
-        else:
-            # Tricky.  Will have to create a Seq object because of the stride
+            # Will have to extract the sequence because of the stride
             full = self.adaptor.get_subseq_as_string(
-                self.primary_id, self.start + i, self.start + j
+                self.primary_id, self.start + start, self.start + end
             )
-            return Seq(full[:: index.step])
-
-    def tostring(self):
-        """Return the full sequence as a python string (DEPRECATED).
-
-        You are now encouraged to use str(my_seq) instead of
-        my_seq.tostring().
-        """
-        import warnings
-
-        warnings.warn(
-            "This method is obsolete; please use str(my_seq) "
-            "instead of my_seq.tostring().",
-            PendingDeprecationWarning,
-        )
-        return self.adaptor.get_subseq_as_string(
-            self.primary_id, self.start, self.start + self._length
-        )
-
-    def __str__(self):
-        """Return the full sequence as a python string."""
-        return self.adaptor.get_subseq_as_string(
-            self.primary_id, self.start, self.start + self._length
-        )
-
-    data = property(tostring, doc="Sequence as string (DEPRECATED)")
-
-    def toseq(self):
-        """Return the full sequence as a Seq object."""
-        # Note - the method name copies that of the MutableSeq object
-        return Seq(str(self))
-
-    def __add__(self, other):
-        """Add another sequence or string to this sequence.
-
-        The sequence is first converted to a Seq object before the addition.
-        The returned object is a Seq object, not a DBSeq object.
-        """
-        return self.toseq() + other
-
-    def __radd__(self, other):
-        """Add another sequence or string to the left.
-
-        The sequence is first converted to a Seq object before the addition.
-        The returned object is a Seq object, not a DBSeq object.
-        """
-        return other + self.toseq()
-
-    def __mul__(self, other):
-        """Multiply sequence by an integer.
-
-        The sequence is first converted to a Seq object before multiplication.
-        The returned object is a Seq object, not a DBSeq object.
-        """
-        return self.toseq() * other
-
-    def __rmul__(self, other):
-        """Multiply integer by a sequence.
-
-        The sequence is first converted to a Seq object before multiplication.
-        The returned object is a Seq object, not a DBSeq object.
-        """
-        return other * self.toseq()
-
-    def __imul__(self, other):
-        """Multiply sequence by integer in-place.
-
-        The sequence is first converted to a Seq object before multiplication.
-        The returned object is a Seq object, not a DBSeq object.
-        """
-        return self.toseq() * other
+            return full[::step].encode("ASCII")
 
 
 def _retrieve_seq_len(adaptor, primary_id):
@@ -189,7 +103,7 @@ def _retrieve_seq(adaptor, primary_id):
     # The database schema ensures there will be only one matching
     # row in the table.
 
-    # If an UnknownSeq was recorded, seq will be NULL,
+    # If an undefined sequence was recorded, seq will be NULL,
     # but length will be populated.  This means length(seq)
     # will return None.
     seqs = adaptor.execute_and_fetchall(
@@ -203,7 +117,7 @@ def _retrieve_seq(adaptor, primary_id):
 
     try:
         length = int(length)
-        given_length = int(length)
+        given_length = int(given_length)
         assert length == given_length
         have_seq = True
     except TypeError:
@@ -221,15 +135,10 @@ def _retrieve_seq(adaptor, primary_id):
     del given_length
 
     if have_seq:
-        return DBSeq(primary_id, adaptor, alphabet=None, start=0, length=int(length))
+        data = _BioSQLSequenceData(primary_id, adaptor, start=0, length=length)
+        return Seq(data)
     else:
-        if moltype in ("dna", "rna"):
-            character = "N"
-        elif moltype == "protein":
-            character = "X"
-        else:
-            character = "?"
-        return UnknownSeq(length, character=character)
+        return Seq(None, length=length)
 
 
 def _retrieve_dbxrefs(adaptor, primary_id):
